@@ -1,6 +1,6 @@
 /*
  * factory.c -- the factory method interfaces
- * Copyright (C) 2003-2017 Meltytech, LLC
+ * Copyright (C) 2003-2019 Meltytech, LLC
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -27,6 +27,7 @@
 extern mlt_consumer consumer_avformat_init( mlt_profile profile, char *file );
 extern mlt_filter filter_avcolour_space_init( void *arg );
 extern mlt_filter filter_avdeinterlace_init( void *arg );
+extern mlt_filter filter_swresample_init( mlt_profile profile, char *arg );
 extern mlt_filter filter_swscale_init( mlt_profile profile, char *arg );
 extern mlt_producer producer_avformat_init( mlt_profile profile, const char *service, char *file );
 extern mlt_filter filter_avfilter_init( mlt_profile, mlt_service_type, const char*, char* );
@@ -126,6 +127,10 @@ static void *create_service( mlt_profile profile, mlt_service_type type, const c
 		return filter_avdeinterlace_init( arg );
 	if ( !strcmp( id, "swscale" ) )
 		return filter_swscale_init( profile, arg );
+#endif
+#ifdef SWRESAMPLE
+	if ( !strcmp( id, "swresample" ) )
+		return filter_swresample_init( profile, arg );
 #endif
 	return NULL;
 }
@@ -208,7 +213,7 @@ static void add_parameters( mlt_properties params, void *object, int req_flags,
 					mlt_properties_set_int( p, "minimum", (int) opt->min );
 				if ( opt->max != INT_MAX )
 					mlt_properties_set_int( p, "maximum", (int) opt->max );
-				mlt_properties_set_int( p, "default", (int) opt->default_val.dbl );
+				mlt_properties_set_int( p, "default", (int) opt->default_val.i64 );
 			}
 			else
 			{
@@ -223,7 +228,7 @@ static void add_parameters( mlt_properties params, void *object, int req_flags,
 				mlt_properties_set_int64( p, "minimum", (int64_t) opt->min );
 			if ( opt->max != INT64_MAX )
 			mlt_properties_set_int64( p, "maximum", (int64_t) opt->max );
-			mlt_properties_set_int64( p, "default", (int64_t) opt->default_val.dbl );
+			mlt_properties_set_int64( p, "default", (int64_t) opt->default_val.i64 );
 			break;
 		case AV_OPT_TYPE_FLOAT:
 			mlt_properties_set( p, "type", "float" );
@@ -385,6 +390,27 @@ static mlt_properties avfilter_metadata( mlt_service_type type, const char *id,
 			mlt_properties_set_int( p, "minimum", 0 );
 			mlt_properties_set_int( p, "default", 0 );
 		}
+		{
+			mlt_properties p = mlt_properties_new();
+			char key[20];
+			int i = 0;
+			snprintf( key, 20, "%d", mlt_properties_count( params ) );
+			mlt_properties_set_data( params, key, p, 0, (mlt_destructor) mlt_properties_close, NULL );
+			mlt_properties_set( p, "identifier", "position" );
+			mlt_properties_set( p, "description", "The MLT position value to set on avfilter frames" );
+			mlt_properties_set( p, "type", "string" );
+			mlt_properties_set( p, "default", "frame" );
+			mlt_properties values = mlt_properties_new();
+			mlt_properties_set_data( p, "values", values, 0, (mlt_destructor) mlt_properties_close, NULL );
+			snprintf( key, 20, "%d", i++ );
+			mlt_properties_set( values, key, "frame" );
+			snprintf( key, 20, "%d", i++ );
+			mlt_properties_set( values, key, "filter" );
+			snprintf( key, 20, "%d", i++ );
+			mlt_properties_set( values, key, "source" );
+			snprintf( key, 20, "%d", i++ );
+			mlt_properties_set( values, key, "producer" );
+		}
 	}
 
 	return metadata;
@@ -428,4 +454,7 @@ MLT_REPOSITORY
 	mlt_properties_close( blacklist );
 #endif // AVFILTER
 #endif
+#ifdef SWRESAMPLE
+	MLT_REGISTER( filter_type, "swresample", create_service );
+#endif
 }
