@@ -1,6 +1,6 @@
 /*
  * producer_xml.c -- a libxml2 parser of mlt service networks
- * Copyright (C) 2003-2018 Meltytech, LLC
+ * Copyright (C) 2003-2019 Meltytech, LLC
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -189,6 +189,58 @@ static void track_service( mlt_properties properties, void *service, mlt_destruc
 	mlt_properties_set_int( properties, "registered", ++ registered );
 }
 
+static inline int is_known_prefix(const char* resource)
+{
+	char *prefix = strchr(resource, ':');
+	if (prefix) {
+		const char *whitelist[] = {
+			"alsa",
+			"avfoundation",
+			"dshow",
+			"fbdev",
+			"gdigrab",
+			"jack",
+			"lavfi",
+			"oss",
+			"pulse",
+			"sndio",
+			"video4linux2",
+			"v4l2",
+			"x11grab",
+			"async",
+			"cache",
+			"concat",
+			"crypto",
+			"data",
+			"ffrtmphttp",
+			"file",
+			"ftp",
+			"gopher",
+			"hls",
+			"http",
+			"httpproxy",
+			"mmsh",
+			"mmst",
+			"pipe",
+			"rtmp",
+			"rtmpt",
+			"rtp",
+			"srtp",
+			"subfile",
+			"tcp",
+			"udp",
+			"udplite",
+			"unix"
+		};
+		size_t i, n = prefix - resource;
+		for (i = 0; i < sizeof(whitelist) / sizeof(whitelist[0]); ++i) {
+			if (!strncmp(whitelist[i], resource, n))
+				return 1;
+		}
+	}
+	return 0;
+}
+
 // Prepend the property value with the document root
 static inline void qualify_property( deserialise_context context, mlt_properties properties, const char *name )
 {
@@ -208,7 +260,9 @@ static inline void qualify_property( deserialise_context context, mlt_properties
 		if ( root != NULL && strcmp( root, "" ) )
 		{
 			char *full_resource = calloc( 1, n );
-			if ( resource[0] != '/' && resource[0] != '\\' && !strchr( resource, ':' ) )
+			int drive_letter = strlen(resource) > 3 && resource[1] == ':' &&
+				(resource[2] == '/' || resource[2] == '\\');
+			if (resource[0] != '/' && resource[0] != '\\' && !drive_letter && !is_known_prefix(resource))
 			{
 				if ( prefix_size )
 					strncat( full_resource, resource_orig, prefix_size );
@@ -553,12 +607,28 @@ static void on_end_producer( deserialise_context context, const xmlChar *name )
 			char *service_name = trim( mlt_properties_get( properties, "mlt_service" ) );
 			if ( resource )
 			{
-				char *temp = calloc( 1, strlen( service_name ) + strlen( resource ) + 2 );
-				strcat( temp, service_name );
-				strcat( temp, ":" );
-				strcat( temp, resource );
-				producer = MLT_SERVICE( mlt_factory_producer( context->profile, NULL, temp ) );
-				free( temp );
+				// If a document was saved as +INVALID.txt (see below), then ignore the mlt_service and
+				// try to load it just from the resource. This is an attempt to recover the failed
+				// producer in case, for example, a file returns.
+				if (!strcmp("qtext", service_name)) {
+					const char *text = mlt_properties_get( properties, "text" );
+					if (text && !strcmp("INVALID", text)) {
+						service_name = NULL;
+					}
+				} else if (!strcmp("pango", service_name)) {
+					const char *markup = mlt_properties_get( properties, "markup" );
+					if (markup && !strcmp("INVALID", markup)) {
+						service_name = NULL;
+					}
+				}
+				if (service_name) {
+					char *temp = calloc( 1, strlen( service_name ) + strlen( resource ) + 2 );
+					strcat( temp, service_name );
+					strcat( temp, ":" );
+					strcat( temp, resource );
+					producer = MLT_SERVICE( mlt_factory_producer( context->profile, NULL, temp ) );
+					free( temp );
+				}
 			}
 			else
 			{
@@ -569,10 +639,15 @@ static void on_end_producer( deserialise_context context, const xmlChar *name )
 		// Just in case the plugin requested doesn't exist...
 		if ( !producer && resource )
 			producer = MLT_SERVICE( mlt_factory_producer( context->profile, NULL, resource ) );
-		if ( !producer )
+		if ( !producer ) {
 			mlt_log_error( NULL, "[producer_xml] failed to load producer \"%s\"\n", resource );
-		if ( !producer )
 			producer = MLT_SERVICE( mlt_factory_producer( context->profile, NULL, "+INVALID.txt" ) );
+			if (producer) {
+				// Save the original mlt_service for the consumer to serialize it as original.
+				mlt_properties_set( MLT_SERVICE_PROPERTIES(producer), "_xml_mlt_service",
+					mlt_properties_get( properties, "mlt_service" ) );
+			}
+		}
 		if ( !producer )
 			producer = MLT_SERVICE( mlt_factory_producer( context->profile, NULL, "colour:red" ) );
 		if ( !producer )
@@ -1130,6 +1205,9 @@ static void on_end_consumer( deserialise_context context, const xmlChar *name )
 					mlt_properties_set_data( consumer_properties, key, properties, 0,
 						(mlt_destructor) mlt_properties_close, NULL );
 
+					// Pass in / out if provided
+					mlt_properties_pass_list( consumer_properties, properties, "in, out" );
+
 					// Pass along quality and performance properties to the multi consumer and its render thread(s).
 					if ( !context->qglsl )
 					{
